{% extends "!layout.html" %}

{% block linktags %}
    <link rel="shortcut icon" href="/static/favicon.ico" />
    <link rel="stylesheet" type="text/css" href="/static/css/sphinx.css" />
    {{ super() }}
{% endblock %}

{% block rootrellink %}
    <li><a title="mini-buildd Home" href="../"><img class="logo" src="_static/img/icon_48x48.png" alt="mini-buildd logo" /></a></li>
    {{ super() }}
{% endblock %}
