{% extends "mini_buildd/base.html" %}
{% load base %}

{% block page_title %}Error {{ code }}{% endblock %}
{% block page_sub_title %}{{ meaning }}{% endblock %}
{% block mbd_page_footer %} &raquo; error page{% endblock %}

{% block content %}
	<div id="mbd_http_error">
		<div class="box error">
			<h1 class="box-caption error">{{ code }} {{ meaning }}</h1>
			<p>{{ description }}</p>
		</div>
	</div>
{% endblock %}
