{% extends "base.html" %}

{% block mbd_page_footer %}
	<tt>user accounts</tt>
{% endblock %}

{% block content %}
	<div class="box">
		<h1 class="box-caption">{% block title %}{% endblock %}</h1>
		<p>
			{% if user.is_authenticated %}
				<b>{{user}}:</b>
				<a href="/accounts/logout/" title="Logout">Logout</a>
				|
				<a href="/accounts/password/change/">Change password</a>
				|
				<a href="/mini_buildd/accounts/profile/" title="Package subscriptions, user upload key">Manage profile</a>
			{% else %}
				<em>Logged off:</em>
				<a href="/accounts/login/" title="Login">Login</a>
				|
				<a href="/accounts/password/reset/">Reset password</a>
				|
				<a href="/accounts/register/">Register new account</a>
			{% endif %}
		</p>
		<hr />
		{% block reg_content %}{% endblock %}
	</div>
{% endblock %}
