{% extends "mini_buildd/base.html" %}

{% block page_title %}API call{% endblock %}
{% block page_sub_title %}{{ api_cmd.COMMAND }}{% endblock %}
{% block mbd_page_footer %} &raquo; API call: {{ api_cmd.COMMAND }}{% endblock %}
