{% extends "registration/base.html" %}

{% block title %}Reset password{% endblock %}

{% block reg_content %}
	<p>
		<b>In case you forgot your password, you can request a password reset procedure here.</b>
	</p>

	<form method="post" action="">
		{% csrf_token %}
		<table>
			{{ form }}
		</table>
		<p>
			<input type="submit" value="Request password reset" />
		</p>
	</form>
{% endblock %}
