# Dummy; needed so django does not freak out on doc building
INSTALLED_APPS = (
    "django_extensions",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "mini_buildd")

SECRET_KEY = "doc has no key. doc needs no key."
