# -*- coding: utf-8 -*-
#
# This file only exists for realizing the use of 'apydoc.py'
# and is supposed to be removed some time later -- most likely
# as soon as sphinx >= 1.1 is available in Squeeze.
