{% load base %}
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">

<html xmlns="http://www.w3.org/1999/xhtml">
	<head>
		<meta http-equiv="content-type" content="text/html; charset=utf-8" />
		<link rel="shortcut icon" href="/static/favicon.ico" />
		<link rel="stylesheet" type="text/css" href="/static/css/mini-buildd.css" />
		<script src="{{ STATIC_URL }}/admin/js/jquery.js" type="text/javascript"></script>
		<script type="text/javascript">
			{% block localscript %}{% endblock %}
		</script>
		<script type="text/javascript" src="/static/js/mini-buildd.js"></script>
		<title>{% block title %}mini-buildd{% endblock %}: {% block page_title %}{% endblock %}</title>
	</head>
	<body>
		<!-- Header -->
		<div id="mbd_header">
			<div id="mbd_branding">
				<a class="image" title="mini-buildd Home" href="/"><img src="/static/img/icon_48x48.png" alt="mini-buildd logo" /></a>
			</div>
			<div id="mbd_navigation">
				<ul class="left">
					<li class="first"><a href="/admin/">Configure</a></li>
					<li><a href="/mini_buildd/api">API</a></li>
					<li><a href="/mini_buildd/api?command=getdputconf">Dput Config</a></li>
					<li><a href="/mini_buildd/api?command=logcat">Daemon Logs</a></li>
					<li><a href="/doc/index.html">Documentation</a></li>
				</ul>
				<ul class="right">
					<li>
						{% if user.is_authenticated %}
							<a class="logged-in" href="/accounts/login/" title="Manage account">User <b>{{ user.username }}</b></a>
						{% else %}
							<a class="logged-out" href="/accounts/login/" title="Login or register account">Logged off</a>
						{% endif %}
					</li>
				</ul>
			</div>
		</div>

		<!-- Page header -->
		<div id="mbd_page_header">
			{% block page_header %}{% endblock %}
		</div>

		<!-- Messages -->
		<div id="mbd_messages">
			{% block messages %}
				{% if messages %}
					<ul class="messagelist">{% for message in messages %}
						<li{% if message.tags %} class="{{ message.tags }}"{% endif %}>{{ message }}</li>
					{% endfor %}</ul>
				{% endif %}
			{% endblock messages %}
		</div>

		<!-- Content -->
		<div id="mbd_content">
			{% block content %}{% endblock %}
		</div>

		<!-- Footer -->
		<div id="mbd_footer">
			<p><a class="homelink" title="mini-buildd Home" href="/mini_buildd/">{% mbd_version %}</a><span class="page-footer">{% block mbd_page_footer %}{% endblock %}</span></p>
		</div>
	</body>
</html>
