Dear {% if user.get_full_name %}{{ user.get_full_name }}{% else %}{{ user }}{% endif %},

you have requested to reset the password of user "{{ user.username }}" on "{{ domain }}".

You may use this link to actually set a new password:

{{ protocol }}://{{ domain }}/accounts/password/reset/confirm/{{ uid }}-{{ token }}

Good luck!

Your mini-buildd at {{ site.name }}
