{% extends "mini_buildd/api.html" %}

{% block content %}
	<div id="mbd_api">
		<div class="box">
			<h1 class="box-caption">API call: {{ api_cmd.COMMAND }}</h1>
			<pre>{{ api_cmd }}</pre>
		</div>
	</div>
{% endblock %}
