{% load base %}
<!-- Manage subscriptions: repositories, package -->
<div id="mbd_subscription">
	<div>
		<form action="/mini_buildd/api" method="get">
			<div>
				<em>Subscriptions to <b>{% if package %}{{ package }}{% else %}any package{% endif %}</b> in:</em>
				<input type="hidden" name="command" value="subscription" />
				<select name="subscription" title="Limit to a distribution">
					<option value="{{ package }}:">any distribution</option>
					{% for r in repositories %}
						{% mbd_distribution_options r value_prefix=package|add:":" %}
					{% endfor %}
				</select>
				<input type="hidden" name="output" value="referer" />
				<input type="submit" name="action" value="list" title="List package subscriptions" />
				<input type="submit" name="action" value="add" title="Add a package subscription" />
				<input type="submit" name="action" value="remove" title="Remove package subscriptions" />
			</div>
		</form>
	</div>
</div>
