/* New mini-buildd specific style definitions. */

#mbd_django_admin_branding h1
{
	color: #ffffcc;
}

#mbd_django_admin_branding img
{
	float: left;
	margin: 0px 10px 8px 10px;
}

#mbd_django_admin_daemon_status
{
	float: left;
	width: 100%;
	background: url("/static/admin/img/nav-bg-reverse.gif") repeat-x scroll 0 0px white;
	border-bottom: 1px solid #cccccc;
	border-top: 1px solid #cccccc;
	color: #999999;
	font-size: 11px;
	padding: 2px 0px 3px 7px;
}

#mbd_django_admin_daemon_status p
{
	display: inline;
}

#mbd_django_admin_daemon_status span.warning
{
	color: red;
	font-weight: bold;
}

#mbd_django_admin_daemon_status a:link, #mbd_django_admin_daemon_status a:visited
{
	font-size: 11px;
	color: #5b80b2;
}

#mbd_django_admin_daemon_status a:hover
{
	color: #003366;
	text-decoration: none;
}

#mbd_box h1.mini-buildd
{
	background: url("/static/admin/img/default-bg.gif") repeat-x scroll left top #7CA0C7;
	color: white;
	margin: 0;
	font-size: 14px;
	padding: 2px 0px 2px 5px;
}

#mbd_box caption
{
	background: url("/static/img/icon_48x48.png") no-repeat scroll right top;
	background-color: #417690;
	border-left: 1px solid #417690;
	border-right: 1px solid #417690;
	font-size: 11px;
	padding: 5px 0px 4px 4px;
}

#mbd_box .inner-box
{
	margin: 12px;
	border: 1px solid #cccccc;
}

/* Extension of existing (default) style definitions. */

#user-tools {
	font-size: 12px;
	padding: 38px 10px 8px 0px;
	position: absolute;
}

/* "mini-buildd"-App Caption */
#header
{
	font-size: 14px;
	background: url("/static/img/da_header_background_gradient.png") repeat-x scroll left top;
}

#header a
{
	font-size: 14px;
}

#content-main table
{
	width: 100%;
}

#content-main caption
{
	font-size: 11px;
	padding: 5px 0px 4px 4px;
}

/* "Auth"-App Caption */
#content-main caption.mb
{
	font-size: 14px;
	padding: 2px 0px 2px 5px;
}

#content-main td.right
{
	text-align: right;
}

#content-main a.changelink, #content-main a.addlink
{
	padding-right: 1px;
	display: inline;
}

#content-main a.wizard
{
	background: url("/static/img/mbd_wizard.png") no-repeat scroll 3px 50%;
	margin: 0px 0px 0px 0px;
	padding: 2px 6px 2px 30px;
	text-decoration: none;
	font-size: 11px;
	font-weight: normal;
	display: inline;
}

#content-main .module.mb
{
	margin-top: 12px;
}

#content-main th.overview
{
	width: 100%;
}

.Prepare {
	border-color: yellow;
	border-width: 3px;
	font-weight: bold;
}

.Check {
	border-color: blue;
	border-width: 3px;
	font-weight: bold;
}

.PC {
	border-color: darkblue;
	border-width: 3px;
	font-weight: bold;
}

.Activate {
	border-color: green;
	border-width: 3px;
	font-weight: bold;
}

.PCA {
	border-color: darkgreen;
	border-width: 3px;
	font-weight: bold;
}

.Deactivate {
	border-color: yellow;
	border-width: 3px;
	font-weight: bold;
}

.Remove {
	border-color: red;
	border-width: 3px;
	font-weight: bold;
}

/* MESSAGES & ERRORS */
ul.messagelist
{
	padding: 5px 0 5px 0;
	margin: 0;
}

ul.messagelist li
{
	font-size: 12px;
	display: block;
	padding: 4px 5px 4px 25px;
	margin: 0 0 3px 0;
	border-bottom: 1px solid #ddd;
	color: #666;
	background: #ffc url(/static/admin/img/icon_success.gif) 5px .3em no-repeat;
}

ul.messagelist li.debug
{
	background-image: url(/static/admin/img/tool-right_over.gif);
	color: blue;
}

ul.messagelist li.info
{
	background-image: url(/static/admin/img/icon_success.gif);
	color: green;
}

ul.messagelist li.warning
{
	background-image: url(/static/admin/img/icon_alert.gif);
	color: purple;
}

ul.messagelist li.error
{
	background-image: url(/static/admin/img/icon_error.gif);
	color: red;
}
