{% extends "registration/base.html" %}

{% block title %}Registration requested{% endblock %}

{% block reg_content %}
	<p>
		To <b>complete</b> your registration, use the
		<b>activation link</b> in the <b>e-mail</b> just sent to you.
	</p>
{% endblock %}
