{% extends "registration/base.html" %}

{% block title %}Password reset{% endblock %}

{% block reg_content %}
	<p>
		Please follow the instructions in the e-mail just sent to you.
	</p>
{% endblock %}
